﻿#region Using directives

using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;

#endregion

namespace MultiSDISample {

  partial class TopLevelForm : Form {

    private string fileName;
    
    public static TopLevelForm CreateTopLevelWindow(string fileName) {

      // Detect if file is already open
      if( !string.IsNullOrEmpty(fileName) ) {
        foreach( TopLevelForm openForm in Application.OpenForms ) {
          if( string.Compare(openForm.FileName, fileName, true) == 0 ) {
            // Bring form to top
            openForm.Activate();
            return openForm;
          }
        }
      }
       
      // Create new top-level form and open file
      TopLevelForm form = new TopLevelForm();
      form.OpenFile(fileName);
      form.Show();
      // Bring form to top
      form.Activate();
      return form;
    }

    static int formCount = 0;
    
    public TopLevelForm() {
      InitializeComponent();

      // Set form count
      ++formCount;
      this.Text += ": " + formCount.ToString();

      // Add new top-level form to the application context
      MultiSDIApplication.Application.AddTopLevelForm(this);

      // Add Window ToolStripMenuItem to the application context
      MultiSDIApplication.Application.AddWindowMenu(this.windowToolStripMenuItem);
    }

    public void OpenFile(string fileName) {
      this.fileName = fileName;
      if( !string.IsNullOrEmpty(fileName) ) {
        using( StreamReader reader = new StreamReader(fileName) ) {
          textBox.Text = reader.ReadToEnd();
        }
      }
      else this.fileName = "Untitled" + formCount.ToString();
      this.Text = this.Text + " (" + this.fileName + ")";
    }

    string FileName {
      get { return this.fileName; }
    }
    
    private void newWindowToolStripMenuItem_Click(object sender, EventArgs e) {
      TopLevelForm.CreateTopLevelWindow(null);
    }

    private void exitWindowToolStripMenuItem_Click(object sender, EventArgs e) {
      this.Close();
    }

    private void openToolStripMenuItem_Click(object sender, EventArgs e) {
      if( this.openFileDialog.ShowDialog() == DialogResult.OK ) {
        TopLevelForm.CreateTopLevelWindow(this.openFileDialog.FileName);
      }
    }
  }
}