using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DataBindingFundamentalsSample {
  public partial class ComplexBindingDataGridViewForm : Form {

    // Create strongly-typed list data source with list management
    // and change notification
    // Create strongly-typed list data source 
    BindingList<RaceCarDriver> raceCarDrivers = new BindingList<RaceCarDriver>();

    public ComplexBindingDataGridViewForm() {
      InitializeComponent();

      // Populate list data source with data items
      this.raceCarDrivers.Add(new RaceCarDriver("M. Schumacher", 500));
      this.raceCarDrivers.Add(new RaceCarDriver("A. Senna", 1000));
      this.raceCarDrivers.Add(new RaceCarDriver("A. Prost", 400));

      this.racingCarDriversDataGridView.DataSource = this.raceCarDrivers;

      // Setup currency information handling
      RefreshItems();
    }

    BindingManagerBase BindingManager {
      get { return this.BindingContext[this.raceCarDrivers]; }
    }
    
    private void moveFirstButton_Click(object sender, EventArgs e) {
      this.BindingManager.Position = 0;
      RefreshItems();
    }

    private void movePreviousButton_Click(object sender, EventArgs e) {
      // No need to worry about being < 0
      --this.BindingManager.Position;
      RefreshItems();
    }

    private void moveNextButton_Click(object sender, EventArgs e) {
      // No need to worry about being > this.BindingManager.Count
      ++this.BindingManager.Position;
      RefreshItems();
    }

    private void moveLastButton_Click(object sender, EventArgs e) {
      this.BindingManager.Position = this.BindingManager.Count - 1;
      RefreshItems();
    }

    void RefreshItems() {
      int count = this.BindingManager.Count;
      int position = this.BindingManager.Position + 1;

      // Update count and position text
      this.countLabel.Text = count.ToString();
      this.positionLabel.Text = position.ToString();

      // Enable/Disable move buttons
      this.moveFirstButton.Enabled = (position > 1);
      this.movePreviousButton.Enabled = (position > 1);
      this.moveNextButton.Enabled = (position < count);
      this.moveLastButton.Enabled = (position < count);
    }

    private void addButton_Click(object sender, EventArgs e) {
      // Add item to list data source directly
      RaceCarDriver raceCarDriver = new RaceCarDriver("Nelson Piquet", 300);
      this.raceCarDrivers.Add(raceCarDriver);
      // Select new item
      this.BindingManager.Position = this.BindingManager.Count - 1;
    }

    private void deleteButton_Click(object sender, EventArgs e) {
      // Remove item from list data source directly
      this.raceCarDrivers.Remove((RaceCarDriver)this.BindingManager.Current);
    }

    private void updateCurrentButton_Click(object sender, EventArgs e) {
      int current = this.BindingManager.Position;
      ((RaceCarDriver)this.raceCarDrivers[0]).Wins++;
    }
  }
}