using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace AppliedDataBindingSample {
  public partial class ItemUIForm : Form {
    public ItemUIForm() {
      InitializeComponent();

      this.unitPriceTextBox.DataBindings["Text"].Format += new ConvertEventHandler(unitPriceTextBox_Format);
      this.unitPriceTextBox.DataBindings["Text"].Parse += new ConvertEventHandler(unitPriceTextBox_Parse);

    }

    void unitPriceTextBox_Format(object sender, ConvertEventArgs e) {
      // Bail if data source's property is null
      if( e.Value == DBNull.Value ) return;

      // Format data source value and concatenate with " Banana"
      // and pluralize if necessary
      string unitPrice = string.Format("{0:0.00 Banana}", e.Value);
      if( (decimal)e.Value != 1 ) unitPrice += "s";
      e.Value = unitPrice;
    }

    void unitPriceTextBox_Parse(object sender, ConvertEventArgs e) {
      // Bail if value not entered
      if( string.IsNullOrEmpty((string)e.Value) ) return;

      // Extract first number from value and convert to decimal
      string unitPrice = (string)e.Value;
      Match match = Regex.Match(unitPrice, @"\d+(.\d{1,2})?");
      e.Value = decimal.Parse(match.Value);
    }

    private void productsBindingNavigatorSaveItem_Click(object sender, EventArgs e) {
      this.Validate();
      this.productsBindingSource.EndEdit();
      this.productsTableAdapter.Update(this.northwindDataSet.Products);

    }

    private void ItemUIForm_Load(object sender, EventArgs e) {
      // TODO: This line of code loads data into the 'northwindDataSet.Suppliers' table. You can move, or remove it, as needed.
      this.suppliersTableAdapter.Fill(this.northwindDataSet.Suppliers);
      // TODO: This line of code loads data into the 'northwindDataSet.Products' table. You can move, or remove it, as needed.
      this.productsTableAdapter.Fill(this.northwindDataSet.Products);
    }

    void unitPriceTextBox_Validating(object sender, CancelEventArgs e) {
      // Check if unit price is a number
      string unitPrice = this.unitPriceTextBox.Text;
      Match match = Regex.Match(unitPrice, @"\d+(.\d{1,2})?");

      // If not correctly formatted, show error and 
      // prevent focus leaving cell
      decimal result;
      if( !decimal.TryParse(match.Value, out result) ) {
        MessageBox.Show("Unit Price must be in this format: 0.00 Bananas");
        e.Cancel = true;
      }
    }
  }
}