using System;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Text;

namespace AppliedDataBindingSample {

  /// <summary>
  /// RaceCarDriver stores information about a Formula 1 race car driver.
  /// </summary>
  public class RaceCarDriver : INotifyPropertyChanged {

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    private void OnPropertyChanged(string propertyName) {
      if( this.PropertyChanged != null ) {
        this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }

    #endregion

    private string name;
    private int wins;
    private DateTime created;
    private Positions positions;

    // Needed so DGV can add new rows
    public RaceCarDriver() { 
      this.created = DateTime.Now;
      positions = new Positions();
      positions.PropertyChanged += positions_PropertyChanged;
    }

    public RaceCarDriver(string name, int wins)
      : this() {
      this.name = name;
      this.wins = wins;
    }

    public RaceCarDriver(string name, int wins, Positions positions) : this(name, wins) {
      this.positions = positions;
    }

    public int Wins {
      get { return this.wins; }
      set {
        this.wins = value;
        this.OnPropertyChanged("Wins");
      }
    }

    public string Name {
      get { return this.name; }
      set {
        this.name = value;
        this.OnPropertyChanged("Name");
      }
    }

    public Positions Positions {
      get { return this.positions; }
      set {
        this.positions = value;
        this.OnPropertyChanged("Positions");
      }
    }

    // Not for data binding
    public DateTime Created {
      get { return this.created; }
      set { this.created = value; }
    }

    private void positions_PropertyChanged(object sender, PropertyChangedEventArgs e) {
      OnPropertyChanged("Positions");
    }
  }
}
